# `branch_name` and `tag_name` methods in this class calls
# `Gitlab::Util.shellout_stdout` which uses `mixlib/shellout` under the hood.
# However, we can't require `mixlib/shellout` in `Gitlab::Util` since it is
# required by `Gitlab::Version`, which is in turn required by Gemfile. So, when
# Gemfile is being evaluated, `mixlib/shellout` will become required, but it
# won't be present in the bundle yet. Hence, we require it here, to demonstrate
# the transitive dependency requirement this class has on `mixlib/shellout`.
require 'mixlib/shellout'

require_relative '../../util'
require_relative '../check'
require_relative 'ci'

module Build
  class Info
    class Git
      class << self
        def branch_name
          # If on CI, branch name from `CI_COMMIT_BRANCH` wins
          result = Build::Info::CI.branch_name

          return result if result

          # If not on CI, attempt to detect branch name
          head_reference = Gitlab::Util.shellout_stdout('git rev-parse --abbrev-ref HEAD')

          # On tags, the shell command will return `HEAD`. If that is not the
          # case, we are on a branch and can return the output we received.
          return head_reference unless head_reference == "HEAD"
        end

        def tag_name
          Build::Info::CI.tag_name || Gitlab::Util.shellout_stdout('git describe --tags --exact-match')
        rescue Gitlab::Util::ShellOutExecutionError => e
          return nil if /fatal: no tag exactly matches/.match?(e.stderr)

          raise "#{e.message}\nSTDOUT: #{e.stdout}\nSTDERR: #{e.stderr}"
        end

        def commit_sha
          commit_sha_raw = Gitlab::Util.get_env('CI_COMMIT_SHA') || Gitlab::Util.shellout_stdout('git rev-parse HEAD')

          commit_sha_raw[0, 8]
        end

        # TODO, merge latest_tag with latest_stable_tag
        # TODO, add tests, needs a repo clone
        def latest_tag
          '17.0.0+ee.0'
        end

        def latest_stable_tag(level: 1)
          '17.0.0+ee.0'
        end
      end
    end
  end
end
